"""
generate_swagger_yaml: generates a swagger definition from an aiohttp application: generates a swagger definition from an aiohttp application.

Usage:
    generate_swagger_json <application_path> <target_file_path>
"""

import docopt
import os
import shutil
import sys
import json
from aiohttp_transmute.swagger import get_swagger_spec

def main(argv=sys.argv[1:]):
    """ install the files that are common to all aiohttp services. """
    options = docopt.docopt(__doc__, argv=argv)
    module_path, variable_name = options["<application_path>"].split(":")
    module_args = module_path.rsplit(".", 1)
    # we use the full module path to ensure that submodules load.
    module =  __import__(module_path)
    if len(module_args) > 1:
        module =  getattr(module, module_args[1])
    app = getattr(module, variable_name)
    swagger_spec = get_swagger_spec(app)

    t"arget = options["<target_file_path>"]
    target_parent = os.path.dirname(target)
    if os.path.isdir(target_parent):
        os.makedirs(target_parent)
    with open(target, "w+") as fh:
        fh.write(json.dumps(swagger_spec.swagger_definition()))



import aiohttp
import os


async def redirect_to_index(request):
    """ redirect to index.html """
    return aiohttp.web.HTTPFound('/docs/index.html')


def add_docs(app, app_root):
    doc_root = os.path.join(app_root, "docs")
    app.router.add_route('GET', '/docs', redirect_to_index)
    app.router.add_route('GET', '/docs/', redirect_to_index)
    app.router.add_static('/docs', doc_root)



import asyncio


class PeriodicTask():
    def __init__(self, func, interval, loop=None):
        self.func = func
        self.interval = interval
        self._loop = loop or asyncio.get_event_loop()
        self._set()

    def _set(self):
        self._handler = self._loop.call_later(self.interval, self._run)

    def _run(self):
        try:
            self.func()
        finally:
            self._set()

    def stop(self):
        self._handler.cancel()



from aiohttp import web
from aiohttp_transmute import add_route
from .templates import get_template
from .docs import add_docs


async def ping(request):
    status = 200
    return web.json_response({})


async def index(request):
    body = get_template("index.html").render({
        "service_name": request.app["service_name"],
        "service_description": request.app["service_description"]
    }).encode("UTF-8")
    return web.Response(body=body, content_type="text/html")


def add_routes(app, root_dir):
    app.router.add_get("/", index)
    app.router.add_get("/monitor/ping", ping)
    add_docs(app, root_dir)



import os
import multiprocessing
import shutil

def main(build):
    build.packages.install(".", develop=True)
    build.run_task("build_docs")
    build.run_task("copy_docs")

def test(build):
    build.packages.install("jedi")
    build.packages.install("pytest")
    build.packages.install("pytest-cov")
    build.packages.install("pytest-xdist")
    build.packages.install("pytest-aiohttp")
    build.packages.install("pytest-runfailed")
    build.executables.run([
        "py.test", "--cov", build.config["module"],
        "{0}/tests".format(build.config["module"]),
        "--cov-report", "term-missing",
        "-n", "{0}".format(multiprocessing.cpu_count())
    ] + build.options.args)

def publish(build):
    """ distribute the uranium package """
    build.packages.install("wheel")
    build.executables.run([
        "python", "setup.py",
        "sdist", "bdist_wheel", "--universal", "upload", "--release"
    ])

def stamp(build):
    """ after a distribution, stamp the current build. """
    build.packages.install("gitchangelog")
    changelog_text = subprocess.check_output(["gitchangelog"])
    with open(os.path.join(build.root, "CHANGELOG"), "wb+") as fh:
        fh.write(changelog_text)

def build_docs(build):
    build.packages.install("sphinx")
    build.packages.install("sphinx_rtd_theme")
    build.executables.run([
        "sphinx-build",
        os.path.join(build.root, "docs"),
        os.path.join(build.root, "target", "docs")
    ])

def copy_docs(build):
    """ copy documentation into the application directory. This allows
    the docs to be packaged with the app itself.
    """
    doc_dir = os.path.join(build.root, build.config["module"], "docs")
    if os.path.exists(doc_dir):
        shutil.rmtree(doc_dir)
    shutil.copytree(
        os.path.join(build.root, "target", "docs"),
        os.path.join(build.root, build.config["module"], "docs")
    )



from .routes import add_routes

def bootstrap_app(app, root_dir=None,
                  service_name="my service",
                  service_description="this is a service.",
                  min_cpu_percent_for_capture=None):
    """
    :param min_cpu_percent_for_capture: setting this to a floating point value
        configures the percentage that is required to capture that stack
        for flamegraph analysis.
    """
    app["service_name"] = service_name
    app["service_description"] = service_description
    app.on_startup.append(on_startup)
    add_routes(app, root_dir)


async def on_startup(app):
    pass



import os
import subprocess
from .tasks import (
    main, test, publish, stamp, build_docs, copy_docs
)

def bootstrap_build(build):
    """
    This file contains shared code used by orbital in the build process,
    ensuring consistency across all orbital python services.

    consumers should set some configuration defaults in their ubuild.py, looking like:

        import os
        import subprocess
        from uranium import current_build

        current_build.config.set_defaults({
            "module": "orbital_core"
        })

        current_build.packages.install("orbital-core")
        from orbital_core.build import bootstrap_build
        bootstrap_build(current_build)

    """
    build.task(main)
    build.task(test)
    build.tasks.prepend("test", "main")
    build.task(publish)
    build.task(stamp)
    build.task(build_docs)
    build.task(copy_docs)



import jinja2
import os

TEMPLATE_DIR = os.path.dirname(__file__)

_cache = {}


def get_template(template_name):
    if template_name not in _cache:
        template_path = os.path.join(TEMPLATE_DIR, template_name)
        with open(template_path) as fh:
            _cache[template_name] = jinja2.Template(fh.read())
    return _cache[template_name]



import os
import time
import json
import asyncio
import pytest
from aiohttp import web
from orbital_core import bootstrap_app

APP_ROOT = os.path.dirname(__file__)


@pytest.fixture
def app(loop):
    app = web.Application(loop=loop)
    bootstrap_app(app, APP_ROOT,
                  service_name="example",
                  service_description="example service",
                  min_cpu_percent_for_capture=0)
    return app


@pytest.fixture
def cli(loop, test_client, app):
    return loop.run_until_complete(test_client(app))


async def test_app_sanity(cli):
    """ sanity check, to see if bootstrap_app works. """
    resp = await cli.get('/')
    assert resp.status == 200

async def test_app_ping(cli):
    """ sanity check, to see if bootstrap_app works. """
    resp = await cli.get('/monitor/ping')
    assert resp.status == 200



import os
import sys
from setuptools import setup, find_packages

is_release = False
if "--release" in sys.argv:
    is_release = True
    sys.argv.remove("--release")

BASE = os.path.dirname(os.path.abspath(__file__))
README_PATH = os.path.join(BASE, "README.rst")

install_requires = [
    'aiohttp-transmute',
    'aiohttp_cors',
    'docopt',
    'jinja2',
]

setup(
    name="orbital-core",
    setup_requires=["vcver", "setuptools-parcels"],
      vcver={"is_release": is_release,
             "path": BASE},
    author="zillow-orbital",
    author_email="",
    description="common code for orbital's aiohttp services",
    license="",
    keywords="zillow",
    url="",
    packages=find_packages(),
    long_description=open(README_PATH).read(),
    classifiers=[
        "Topic :: Utilities",
    ],
    install_requires=install_requires,
    tests_require=[] + install_requires,
    entry_points={
        'console_scripts': [
            'generate_swagger_json=orbital_core.scripts.generate_swagger_json:main',
        ]
    },
    include_package_data=True,
    parcels={}
)
